﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using WpfAudioPlayer.Models.Wmf;
using System.Globalization;

namespace WpfAudioPlayer.Models
{
	/// <summary>
	/// 曲を表します。
	/// </summary>
	class Song
	{
		/// <summary>
		/// 音楽ファイルから Song インスタンスを生成します。
		/// </summary>
		/// <param name="fileName">音楽ファイル。</param>
		/// <returns>Song インタンス。</returns>
		public static Song FromFile( string fileName )
		{
			if( String.IsNullOrEmpty( fileName ) ) { throw new ArgumentException( "fileName" ); }

			var song = new Song() { FilePath = fileName, Title = Path.GetFileNameWithoutExtension( fileName ) };

			using( var editor = new WmfTagEditor( fileName ) )
			{
				var value = editor.Read( TagType.Title );
				if( value != null ) { song.Title = value.String; }

				value = editor.Read( TagType.TrackNumber );
				if( value != null ) { song.TrackNumber = value.Int32; }
				
				value = editor.Read( TagType.AlbumTitle );
				if( value != null ) { song.Album = value.String; }
	
				value = editor.Read( TagType.AlbumArtist );
				if( value != null ) { song.AlbumArtist = value.String; }

				value = editor.Read( TagType.Artist );
				if( value != null ) { song.Artist = value.String; }
				
				value = editor.Read( TagType.Duration );
				if( value != null )
				{
					song.Duration = String.Format( CultureInfo.InvariantCulture, "{0:d2}:{1:d2}:{2:d2}", value.TimeSpan.Hours, value.TimeSpan.Minutes, value.TimeSpan.Seconds );
				}

				value = editor.Read( TagType.Genre );
				if( value != null ) { song.Genre = value.String; }
			}

			return song;
		}
		
		/// <summary>
		/// アルバム名を取得します。
		/// </summary>
		public string Album { get; private set; }
	
		/// <summary>
		/// アルバムのアーティスト名を取得します。
		/// </summary>
		public string AlbumArtist { get; private set; }

		/// <summary>
		/// アーティスト名を取得します。
		/// </summary>
		public string Artist { get; private set; }
		
		/// <summary>
		/// 演奏時間を取得します。
		/// </summary>
		public string Duration { get; private set; }

		/// <summary>
		/// ジャンルを取得します。
		/// </summary>
		public string Genre { get; private set; }

		/// <summary>
		/// 曲名を取得します。
		/// </summary>
		public string Title { get; private set; }
		
		/// <summary>
		/// トラック番号を取得します。
		/// </summary>
		public int TrackNumber { get; private set; }

		/// <summary>
		/// 音楽ファイルへのパスを取得します。
		/// </summary>
		public string FilePath { get; private set; }
	}
}
