﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace WpfAudioPlayer.Models.Wmf
{
	/// <summary>
	/// Windows Media のメタデータ編集クラス。
	/// </summary>
	class WmfTagEditor : IDisposable
	{
		/// <summary>
		/// コンストラクタ。
		/// </summary>
		/// <param name="fileLocation">編集対象となるファイルへのパス。</param>
		public WmfTagEditor( string fileName )
		{
			try
			{
				IWMMetadataEditor2 editor = null;
				NativeMethods.WMCreateEditor( out editor );

				editor.Open( fileName );
				IWMHeaderInfo wmHeader = ( IWMHeaderInfo )editor;
				this._wmfHeaderInfo = new WmfHeaderInfo( wmHeader );
				this._fileLocation = fileName;
			}
			catch( COMException exception )
			{
				if( ( ulong )exception.ErrorCode == NativeMethods.NS_E_FILE_OPEN_FAILED )
				{
					throw new System.IO.FileNotFoundException( "ファイルが見つかりません。", fileName );
				}
				else
				{
					throw exception;
				}
			}
		}

		/// <summary>
		/// 文字列のタグ情報を 16 ビット整数のタグ情報へ変換する。
		/// </summary>
		/// <param name="value">文字列のタグ情報。</param>
		/// <returns>変換したタグ情報を返す。</returns>
		private static TagValue StrToInt16Value( string value )
		{
			short shortValue = 0;
			short.TryParse( value, out shortValue );
			return new TagValue( shortValue );
		}

		/// <summary>
		/// 文字列のタグ情報を 32 ビット整数のタグ情報へ変換する。
		/// </summary>
		/// <param name="value">文字列のタグ情報。</param>
		/// <returns>変換したタグ情報を返す。</returns>
		private static TagValue StrToInt32Value( string value )
		{
			int intValue = 0;
			int.TryParse( value, out intValue );
			return new TagValue( intValue );
		}

		/// <summary>
		/// 文字列のタグ情報を日時データのタグ情報へ変換する。
		/// </summary>
		/// <param name="value">文字列のタグ情報。</param>
		/// <returns>変換したタグ情報を返す。</returns>
		private static TagValue StrToDateTimeValue( string value )
		{
			DateTime dateTime;
			if( !DateTime.TryParse( value, out dateTime ) )
			{
				int year = 0;
				dateTime = new DateTime( ( int.TryParse( value, out year ) ? year : 1 ), 1, 1 );
			}

			return new TagValue( dateTime );
		}

		/// <summary>
		/// リソースが解放済みである事を示す値を設定・取得する。
		/// </summary>
		private bool _isDisposed;

		/// <summary>
		/// メタデータを操作する為のインターフェース。
		/// </summary>
		private WmfHeaderInfo _wmfHeaderInfo;

		/// <summary>
		/// 操作対象となるファイルへのパス。
		/// </summary>
		private readonly string _fileLocation;

		/// <summary>
		/// タグ情報を読み取る。
		/// </summary>
		/// <param name="type">タグの種別。</param>
		/// <returns>成功時はタグ情報、失敗時は null を返す。</returns>
		public TagValue Read( TagType type )
		{
			WmfTagValue wmfTagValue = null;

			switch( type )
			{
			case TagType.AlbumArtist:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMAlbumArtist );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.AlbumTitle:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMAlbumTitle );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Artist:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMAuthor );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.ArtistURL:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMAuthorURL );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.BeatsPerMinute:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMBeatsPerMinute );
				return ( wmfTagValue == null ? null : WmfTagEditor.StrToInt16Value( wmfTagValue.String ) );

			case TagType.Comments:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMComments );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Composer:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMComposer );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Conductor:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMConductor );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Duration:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMDuration );
				return ( wmfTagValue == null ? null : new TagValue( new TimeSpan( ( long )wmfTagValue.QWord ) ) );

			case TagType.Genre:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMGenre );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Language:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMLanguage );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.PartOfSet:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMPartOfSet );
				return ( wmfTagValue == null ? null : WmfTagEditor.StrToInt16Value( wmfTagValue.String ) );

			case TagType.Publisher:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMPublisher );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Title:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMTitle );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.TrackNumber:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMTrackNumber );
				if( wmfTagValue == null )
				{
					return null;
				}
				else if( wmfTagValue.Type == WMT_ATTR_DATATYPE.WMT_TYPE_DWORD )
				{
					return new TagValue( ( int )wmfTagValue.DWord );
				}
				else
				{
					return WmfTagEditor.StrToInt32Value( wmfTagValue.String );
				}

			case TagType.Writer:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMWriter );
				return ( wmfTagValue == null ? null : new TagValue( wmfTagValue.String ) );

			case TagType.Year:
				wmfTagValue = this._wmfHeaderInfo.GetTagValue( WmfAttributeNames.WMYear );
				return ( wmfTagValue == null ? null : WmfTagEditor.StrToDateTimeValue( wmfTagValue.String ) );

			default:
				return null;
			}
		}

		#region IDisposable メンバ

		/// <summary>
		/// リソースを解放する。
		/// </summary>
		/// <param name="disposing">マネージドリソースも解放する場合は true、それ以外は false を指定する。</param>
		protected virtual void Dispose( bool disposing )
		{
			if( !this._isDisposed )
			{
				if( disposing )
				{
					// マネージドリソースの解放
				}

				// メタデータ操作用オブジェクトの解放
				if( this._wmfHeaderInfo != null )
				{
					( ( IWMMetadataEditor2 )this._wmfHeaderInfo.HeaderInfo ).Close();
					this._wmfHeaderInfo = null;
				}
			}

			this._isDisposed = true;
		}

		/// <summary>
		/// リソースを解放する。
		/// </summary>
		public void Dispose()
		{
			this.Dispose( true );
			GC.SuppressFinalize( this );
		}

		/// <summary>
		/// デストラクタ。
		/// </summary>
		~WmfTagEditor()
		{
			this.Dispose( false );
		}

		#endregion
	}

	/// <summary>
	/// ファイルのタグ情報を格納する為のクラス。
	/// </summary>
	public class TagValue
	{
		/// <summary>
		/// 文字列データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( string value ) { this.String = value; }

		/// <summary>
		/// 日時データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( DateTime value ) { this.DateTime = value; }

		/// <summary>
		/// 時間データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( TimeSpan value ) { this.TimeSpan = value; }

		/// <summary>
		/// 16 ビット整数データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( short value ) { this.Int16 = value; }

		/// <summary>
		/// 32 ビット整数データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( int value ) { this.Int32 = value; }

		/// <summary>
		/// 64 ビット整数データでインスタンスを初期化する。
		/// </summary>
		/// <param name="value">データ。</param>
		public TagValue( long value ) { this.Int64 = value; }

		/// <summary>
		/// 文字列データを取得する。
		/// </summary>
		public string String { get; private set; }

		/// <summary>
		/// 日時データを取得する。
		/// </summary>
		public DateTime DateTime { get; private set; }

		/// <summary>
		/// 時間データを取得する。
		/// </summary>
		public TimeSpan TimeSpan { get; private set; }

		/// <summary>
		/// 16 ビット整数データを取得する。
		/// </summary>
		public short Int16 { get; private set; }

		/// <summary>
		/// 32 ビット整数データを取得する。
		/// </summary>
		public int Int32 { get; private set; }

		/// <summary>
		/// 64 ビット整数データを取得する。
		/// </summary>
		public long Int64 { get; private set; }
	}

	/// <summary>
	/// タグ情報の種別を定義した列挙体。
	/// </summary>
	public enum TagType
	{
		/// <summary>
		/// アルバムのアーティスト名。
		/// この情報は、文字列となる。
		/// </summary>
		AlbumArtist,

		/// <summary>
		/// アルバム名。
		/// この情報は、文字列となる。
		/// </summary>
		AlbumTitle,

		/// <summary>
		/// アーティスト名。
		/// この情報は、文字列となる。
		/// </summary>
		Artist,

		/// <summary>
		/// アーティストの公式 Web ページへの URL。
		/// この情報は、文字列となる。
		/// </summary>
		ArtistURL,

		/// <summary>
		/// 演奏のテンポ ( 一分間の拍数 )。
		/// この情報は、16 ビット整数データとなる。
		/// </summary>
		BeatsPerMinute,

		/// <summary>
		/// コメント。
		/// この情報は、文字列となる。
		/// </summary>
		Comments,

		/// <summary>
		/// 作曲者。
		/// この情報は、文字列となる。
		/// </summary>
		Composer,

		/// <summary>
		/// 指揮者。
		/// この情報は、文字列となる。
		/// </summary>
		Conductor,

		/// <summary>
		/// 演奏時間。
		/// この情報は、時間データとなる。
		/// </summary>
		Duration,

		/// <summary>
		/// ジャンル。
		/// この情報は、文字列となる。
		/// </summary>
		Genre,

		/// <summary>
		/// 言語。
		/// この情報は、文字列となる。
		/// </summary>
		Language,

		/// <summary>
		/// アルバムが複数枚で構成されている場合、何枚目に収録されているかを示すインデックス。
		/// この情報は、16 ビット整数データとなる。
		/// </summary>
		PartOfSet,

		/// <summary>
		/// 出版社。
		/// この情報は、文字列となる。
		/// </summary>
		Publisher,

		/// <summary>
		/// 曲名。
		/// この情報は、文字列となる。
		/// </summary>
		Title,

		/// <summary>
		/// トラック番号。
		/// この情報は、32 ビット整数データとなる。
		/// </summary>
		TrackNumber,

		/// <summary>
		/// 作詞者。
		/// この情報は、文字列となる。
		/// </summary>
		Writer,

		/// <summary>
		/// リリース年。
		/// この情報は、日時データとなる。
		/// </summary>
		Year
	}
}
